import 'package:flutter/material.dart';
import 'dart:ui';

enum NiceDialogType { NONE, INFO, WARNING, ERROR, MULTIPLE_OPTIONS }

// ignore_for_file: must_be_immutable
class NiceDialog extends StatelessWidget {
  NiceDialogType type;

  Color typeInfoColor;
  Color typeWarningColor;
  Color typeErrorColor;
  Color typeMultipleOptionsColor;

  Color? backgroundColor;
  double? elevation;
  ShapeBorder shape;

  Widget? header;
  Color? headerColor;
  Widget? headerIcon;
  IconData? headerIconData;
  double headerIconSize;
  Color headerIconColor;
  double headerPadding;
  double borderRadius;

  Widget? title;
  String? titleText;
  Color? titleColor;

  Widget? content;
  String? message;
  Color? messageColor;

  Widget? btnOk;
  String? btnOkText;
  Color? btnOkColor;
  VoidCallback? btnOkAction;

  Widget? btnCancel;
  String? btnCancelText;
  Color? btnCancelColor;
  VoidCallback? btnCancelAction;

  Widget? btnNeutral;
  String? btnNeutralText;
  Color? btnNeutralColor;
  VoidCallback? btnNeutralAction;
  double? textScaleFactor;

  NiceDialog({
    NiceDialogType? type,
    Color? typeInfoColor,
    Color? typeWarningColor,
    Color? typeErrorColor,
    Color? typeMultipleOptionsColor,
    this.backgroundColor,
    this.elevation,
    ShapeBorder? shape,
    this.header,
    this.headerColor,
    this.headerIcon,
    this.headerIconData,
    double? headerIconSize,
    Color? headerIconColor,
    double? headerPadding,
    double? borderRadius,
    this.title,
    this.titleText,
    this.titleColor,
    this.content,
    this.message,
    this.messageColor,
    this.btnOk,
    this.btnOkText,
    this.btnOkColor,
    this.btnOkAction,
    this.btnCancel,
    this.btnCancelText,
    this.btnCancelColor,
    this.btnCancelAction,
    this.btnNeutral,
    this.btnNeutralText,
    this.btnNeutralColor,
    this.btnNeutralAction,
    this.textScaleFactor,
  })  : type = type ?? NiceDialogType.NONE,
        typeInfoColor = typeInfoColor ?? Colors.blue,
        typeWarningColor = typeWarningColor ?? Colors.orange,
        typeErrorColor = typeErrorColor ?? const Color(0xFFC62828),
        typeMultipleOptionsColor = typeMultipleOptionsColor ?? Colors.green,
        headerIconSize = headerIconSize ?? 48,
        headerIconColor = headerIconColor ?? Colors.white,
        headerPadding = headerPadding ?? 8,
        borderRadius = borderRadius ?? 16,
        shape = shape ??
            RoundedRectangleBorder(
                borderRadius:
                    BorderRadius.all(Radius.circular(borderRadius ?? 16))) {
    headerIconData ??= (getDialogTypeIconData() ?? Icons.info);
    headerColor ??= (getDialogTypeColor() ?? Colors.blue);
    headerIcon ??= Icon(
      headerIconData,
      color: this.headerIconColor,
      size: this.headerIconSize,
    );
  }

  Color? getDialogTypeColor() {
    switch (type) {
      case NiceDialogType.INFO:
        return typeInfoColor;
      case NiceDialogType.WARNING:
        return typeWarningColor;
      case NiceDialogType.ERROR:
        return typeErrorColor;
      case NiceDialogType.MULTIPLE_OPTIONS:
        return typeMultipleOptionsColor;
      case NiceDialogType.NONE:
      default:
        return null;
    }
  }

  IconData? getDialogTypeIconData() {
    switch (type) {
      case NiceDialogType.INFO:
        return Icons.info;
      case NiceDialogType.WARNING:
        return Icons.warning;
      case NiceDialogType.ERROR:
        return Icons.error;
      case NiceDialogType.MULTIPLE_OPTIONS:
        return Icons.list;
      case NiceDialogType.NONE:
      default:
        return null;
    }
  }

  void checkParams(BuildContext context) {
    titleColor ??= Theme.of(context).textTheme.titleLarge!.color;
    if (title == null && titleText != null) {
      title = Text(
        titleText!,
        textAlign: TextAlign.center,
        style: TextStyle(
            fontSize: 18, color: titleColor, fontWeight: FontWeight.bold),
      );
    }

    messageColor ??= Theme.of(context).textTheme.titleSmall!.color;
    if (content == null && message != null) {
      content = Text(
        message!,
        textAlign: TextAlign.center,
        style: TextStyle(color: messageColor),
      );
    }

    backgroundColor ??= DialogTheme.of(context).backgroundColor;
    elevation ??= DialogTheme.of(context).elevation;
    Color accentColor = Theme.of(context).colorScheme.secondary;
    btnOkColor ??= accentColor;
    btnCancelColor ??= accentColor;
    btnNeutralColor ??= accentColor;
  }

  Widget? buildHeader() {
    if (header != null) return header;
    return Container(
      alignment: Alignment.center,
      padding: EdgeInsets.all(headerPadding),
      decoration: ShapeDecoration(
          color: headerColor,
          shape: RoundedRectangleBorder(
              borderRadius: BorderRadius.only(
                  topLeft: Radius.circular(borderRadius),
                  topRight: Radius.circular(borderRadius)))),
      child: headerIcon,
    );
  }

  Widget buildTitle() {
    return Container(
      margin: const EdgeInsets.only(top: 10),
      child: title,
    );
  }

  Widget buildContent() {
    return Container(
      margin: const EdgeInsets.only(top: 10),
      child: content,
    );
  }

  Widget? buildButtons(BuildContext context) {
    if (btnNeutral == null && btnNeutralText != null) {
      btnNeutral = TextButton(
        style: ButtonStyle(
          foregroundColor: MaterialStateProperty.all<Color>(
              btnNeutralColor ?? Theme.of(context).colorScheme.secondary),
        ),
        onPressed: btnNeutralAction ?? () => Navigator.of(context).pop(),
        child: Text(btnNeutralText!),
      );
    }
    if (btnCancel == null && btnCancelText != null) {
      btnCancel = TextButton(
        style: ButtonStyle(
          foregroundColor: MaterialStateProperty.all<Color>(
              btnCancelColor ?? Theme.of(context).colorScheme.secondary),
        ),
        onPressed: btnCancelAction ?? () => Navigator.of(context).pop(),
        child: Text(btnCancelText!),
      );
    }
    if (btnOk == null && btnOkText != null) {
      btnOk = TextButton(
        style: ButtonStyle(
          foregroundColor: MaterialStateProperty.all<Color>(
              btnOkColor ?? Theme.of(context).colorScheme.secondary),
        ),
        onPressed: btnOkAction ?? () => Navigator.of(context).pop(),
        child: Text(btnOkText!),
      );
    }
    return (btnNeutral != null || btnCancel != null || btnOk != null)
        ? Wrap(
            alignment: WrapAlignment.center,
            spacing: 8,
            children: <Widget>[
              if (btnNeutral != null) btnNeutral!,
              if (btnCancel != null) btnCancel!,
              if (btnOk != null) btnOk!,
            ],
          )
        : null;
  }

  @override
  Widget build(BuildContext context) {
    checkParams(context);
    final buttons = buildButtons(context);
    final dialog = Dialog(
      backgroundColor: backgroundColor,
      elevation: elevation,
      shape: shape,
      child: SizedBox(
        width: MediaQueryData.fromWindow(window).size.shortestSide * 60 / 100,
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: [
            buildHeader()!,
            Flexible(
              child: Scrollbar(
                  child: SingleChildScrollView(
                child: Column(
                  mainAxisSize: MainAxisSize.min,
                  children: <Widget>[
                    Padding(
                      padding: const EdgeInsets.symmetric(horizontal: 16),
                      child: buildTitle(),
                    ),
                    Padding(
                      padding: const EdgeInsets.symmetric(horizontal: 16),
                      child: buildContent(),
                    ),
                  ],
                ),
              )),
            ),
            if (buttons != null)
              Padding(
                padding: const EdgeInsets.only(top: 16),
                child: buttons,
              )
          ],
        ),
      ),
    );
    return textScaleFactor != null
        ? MediaQuery(
            data: MediaQuery.of(context)
                .copyWith(textScaleFactor: textScaleFactor),
            child: dialog,
          )
        : dialog;
  }
}
