import 'package:flutter/material.dart';
import 'package:adeptutils/adeptutils.dart';
import 'dart:async';

typedef OnItemSelected = void Function(
    int? selectedItemId, int? unselectedItemId);

class DrawerWrapper {
  int? selectedItemId;
}

class AdeptDrawer extends Drawer {
  final _wrapper = DrawerWrapper();

  set _selectedItemId(int? selectedItemId) =>
      _wrapper.selectedItemId = selectedItemId;
  int? get selectedItemId => _wrapper.selectedItemId;

  final GlobalKey<ScaffoldState> scaffoldKey;
  final OnItemSelected? onItemSelected;
  final ListStack<int> visitedItemsId = ListStack();

  AdeptDrawer({
    Key? key,
    required this.scaffoldKey,
    this.onItemSelected,
    int? defaultSelectedItemId,
    ListStack<int>? defaultVisitedItemsId,
    double elevation = 16.0,
    Widget? child,
    String? semanticLabel,
  }) : super(
          key: key,
          elevation: elevation,
          child: child,
          semanticLabel: semanticLabel,
        ) {
    if (defaultVisitedItemsId?.isNotEmpty ?? false) {
      visitedItemsId.addAll(defaultVisitedItemsId!);
      if (visitedItemsId.isNotEmpty) _selectedItemId = visitedItemsId.peek();
    } else {
      selectItem(defaultSelectedItemId);
    }
  }

  bool get isDrawerOpen => scaffoldKey.currentState?.isDrawerOpen ?? false;
  bool get isEndDrawerOpen =>
      scaffoldKey.currentState?.isEndDrawerOpen ?? false;
  void openDrawer() => scaffoldKey.currentState?.openDrawer();
  void openEndDrawer() => scaffoldKey.currentState?.openEndDrawer();
  void closeDrawer(BuildContext? context) {
    context ??= scaffoldKey.currentContext;
    if (context != null && isDrawerOpen) Navigator.pop(context);
  }

  void selectItem(int? selectedItemId,
      {bool repeated = false, bool update = true}) {
    if (selectedItemId == null) return;
    int? currentItemId;

    if (visitedItemsId.isNotEmpty) currentItemId = visitedItemsId.peek();
    if (!repeated && currentItemId == selectedItemId) return;

    visitedItemsId.push(_selectedItemId = selectedItemId);
    try {
      if (update && onItemSelected != null) {
        onItemSelected?.call(selectedItemId, currentItemId);
      }
    } catch (e) {
      print(e);
    }
  }

  void _onBackVisit() {
    if (visitedItemsId.size > 1) {
      int? currentItemId = visitedItemsId.pop();
      _selectedItemId = visitedItemsId.peek();
      try {
        if (onItemSelected != null) {
          onItemSelected?.call(selectedItemId, currentItemId);
        }
      } catch (e) {
        print(e);
      }
    }
  }

  /// onBackPressed returns false to stop propagating the event,
  /// true in other cases
  Future<bool> onBackPressed({BuildContext? context}) async {
    if (isDrawerOpen) {
      closeDrawer(context);
      return false;
    }
    _onBackVisit();
    return true;
  }
}
