import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:adeptutils/adeptutils.dart';
import 'dart:async';

class AutoCompleteItem<T> implements Comparable<AutoCompleteItem<T>> {
  final T value;
  final String text;

  AutoCompleteItem({required this.value, required this.text});

  @override
  int compareTo(AutoCompleteItem<T> other) {
    return text.compareTo(other.text);
  }

  @override
  String toString() => text;

  @override
  bool operator ==(Object other) => value == other;
  @override
  int get hashCode => value.hashCode;
  bool endsWith(String other) => text.endsWith(other);
  bool startsWith(Pattern pattern, [int index = 0]) =>
      text.startsWith(pattern, index);
  bool contains(Pattern other, [int startIndex = 0]) =>
      text.contains(other, startIndex);
  String toLowerCase() => text.toLowerCase();
  String toUpperCase() => text.toUpperCase();
}

typedef AutoCompleteOverlayItemBuilder<T> = Widget Function(
    BuildContext context, AutoCompleteItem<T> autoCompleteItem);

typedef Filter<T> = bool Function(
    AutoCompleteItem<T> autoCompleteItem, String query);

typedef InputEventCallback<T> = Function(AutoCompleteItem<T>? autoCompleteItem);

typedef StringCallback = Function(String data);

// ignore_for_file: annotate_overrides, overridden_fields, no_logic_in_create_state
class AutoCompleteDropDownFormField<T> extends StatefulWidget {
  final List<AutoCompleteItem<T>> autoCompleteItems;
  final Filter<T>? itemFilter;
  final Comparator<AutoCompleteItem<T>>? itemSorter;
  final StringCallback? textChanged, textSubmitted;
  final ValueSetter<bool>? onFocusChanged;
  final InputEventCallback<T>? valueChanged;
  final AutoCompleteOverlayItemBuilder<T>? itemBuilder;
  final int? autoCompleteItemsAmount;
  final GlobalKey<AutoCompleteDropDownFormFieldState<T>> key;
  final bool? submitOnAutoCompleteItemTap, clearOnSubmit;
  final List<TextInputFormatter>? inputFormatters;
  final int? minLength;

  final InputDecoration? decoration;
  final TextStyle? style;
  final TextInputType? keyboardType;
  final TextInputAction? textInputAction;
  final TextCapitalization? textCapitalization;
  final AdeptValueController<AutoCompleteItem<T>>? controller;
  final FocusNode? focusNode;

  final double? autoCompleteItemItemHeight;
  final double? autoCompleteItemsBoxMaxHeight;
  final bool? enabled;

  const AutoCompleteDropDownFormField({
    required this.key, //GlobalKey used to enable addSuggestion etc
    required this.autoCompleteItems, //Suggestions that will be displayed
    this.controller,
    this.valueChanged, //Callback on item selected, this is the item selected of type <T>
    this.itemBuilder, //Callback to build each item, return a Widget
    this.itemSorter, //Callback to sort items in the form (a of type <T>, b of type <T>)
    this.itemFilter, //Callback to filter item: return true or false depending on input text
    this.inputFormatters,
    this.style,
    this.decoration,
    this.textChanged, //Callback on input text changed, this is a string
    this.textSubmitted, //Callback on input text submitted, this is also a string
    this.onFocusChanged,
    this.keyboardType,
    this.autoCompleteItemsAmount, //The amount of autoCompleteItems to show, larger values may result in them going off screen
    this.submitOnAutoCompleteItemTap, //Call textSubmitted on autoCompleteItem tap, valueChanged will be called no matter what
    this.clearOnSubmit, //Clear autoCompleteTextfield on submit
    this.textInputAction,
    this.textCapitalization,
    this.minLength,
    this.focusNode,
    this.autoCompleteItemItemHeight,
    this.autoCompleteItemsBoxMaxHeight,
    this.enabled,
  }) : super(key: key);

  void clear() => key.currentState!.clear();

  void addSuggestion(AutoCompleteItem<T> autoCompleteItem) =>
      key.currentState!.addSuggestion(autoCompleteItem);

  void removeSuggestion(AutoCompleteItem<T> autoCompleteItem) =>
      key.currentState!.removeSuggestion(autoCompleteItem);

  void updateSuggestions(List<AutoCompleteItem<T>> autoCompleteItems) =>
      key.currentState!.updateSuggestions(autoCompleteItems);

  void triggerSubmitted() => key.currentState!.triggerSubmitted();

  void updateDecoration(
          {InputDecoration? decoration,
          List<TextInputFormatter>? inputFormatters,
          TextCapitalization? textCapitalization,
          TextStyle? style,
          TextInputType? keyboardType,
          TextInputAction? textInputAction}) =>
      key.currentState!.updateDecoration(
          decoration: decoration,
          inputFormatters: inputFormatters,
          textCapitalization: textCapitalization,
          style: style,
          keyboardType: keyboardType,
          textInputAction: textInputAction);

  TextField? get textField => key.currentState!.textField;

  @override
  State<StatefulWidget> createState() => AutoCompleteDropDownFormFieldState<T>(
        autoCompleteItems,
        textChanged,
        textSubmitted,
        onFocusChanged,
        valueChanged,
        itemBuilder,
        itemSorter,
        itemFilter,
        autoCompleteItemsAmount ?? 5,
        submitOnAutoCompleteItemTap ?? true,
        clearOnSubmit ?? false,
        minLength ?? 1,
        inputFormatters,
        textCapitalization ?? TextCapitalization.sentences,
        decoration ?? const InputDecoration(),
        style,
        keyboardType ?? TextInputType.text,
        textInputAction ?? TextInputAction.done,
        controller,
        focusNode,
        autoCompleteItemItemHeight ?? 56,
        autoCompleteItemsBoxMaxHeight ?? 224,
        enabled ?? true,
      );
}

class AutoCompleteDropDownFormFieldState<T>
    extends State<AutoCompleteDropDownFormField> {
  final LayerLink _layerLink = LayerLink();

  TextField? textField;
  List<AutoCompleteItem<T>> autoCompleteItems;
  StringCallback? textChanged, textSubmitted;
  ValueSetter<bool>? onFocusChanged;
  InputEventCallback<T>? valueChanged;
  AutoCompleteOverlayItemBuilder<T>? itemBuilder;
  Comparator<AutoCompleteItem<T>>? itemSorter;
  OverlayEntry? listAutoCompleteItemsEntry;
  List<AutoCompleteItem<T>>? filteredAutoCompleteItems;
  Filter<T>? itemFilter;
  int autoCompleteItemsAmount;
  int minLength;
  bool submitOnAutoCompleteItemTap, clearOnSubmit;
  AdeptValueController<AutoCompleteItem<T>>? controller;
  AdeptTextEditingController? textEditingController;
  FocusNode? focusNode;

  String currentText = "";

  InputDecoration decoration = const InputDecoration();
  List<TextInputFormatter>? inputFormatters;
  TextCapitalization textCapitalization;
  TextStyle? style;
  TextInputType keyboardType;
  TextInputAction textInputAction;

  double autoCompleteItemItemHeight;
  double autoCompleteItemsBoxMaxHeight;
  bool enabled;

  bool hasText = false;
  bool showingSuggestions = false;

  AutoCompleteDropDownFormFieldState(
    this.autoCompleteItems,
    this.textChanged,
    this.textSubmitted,
    this.onFocusChanged,
    this.valueChanged,
    this.itemBuilder,
    this.itemSorter,
    this.itemFilter,
    this.autoCompleteItemsAmount,
    this.submitOnAutoCompleteItemTap,
    this.clearOnSubmit,
    this.minLength,
    this.inputFormatters,
    this.textCapitalization,
    this.decoration,
    this.style,
    this.keyboardType,
    this.textInputAction,
    this.controller,
    this.focusNode,
    this.autoCompleteItemItemHeight,
    this.autoCompleteItemsBoxMaxHeight,
    this.enabled,
  ) {
    itemBuilder ??= (context, item) {
      return Container(
        height: autoCompleteItemItemHeight,
        alignment: AlignmentDirectional.centerStart,
        child: Text(item.text),
      );
    };
    itemFilter ??= (AutoCompleteItem<T> item, String query) =>
        item.toLowerCase().contains(query.toLowerCase());
    itemSorter ??=
        (AutoCompleteItem<T> a, AutoCompleteItem<T> b) => a.compareTo(b);
    controller ??= AdeptValueController<AutoCompleteItem<T>>();
    textEditingController =
        AdeptTextEditingController(text: controller?.value?.text);
    updateDecoration(refresh: false);

    if (textEditingController?.text != null) {
      currentText = textEditingController!.text;
      hasText = !TextUtils.isEmpty(currentText);
    }

    textField!.focusNode!.addListener(() {
      if (onFocusChanged != null) {
        onFocusChanged!(textField!.focusNode!.hasFocus);
      }

      if (!textField!.focusNode!.hasFocus) {
        filteredAutoCompleteItems = [];
        updateOverlay();
      } else if (currentText.isNotEmpty) {
        updateOverlay(currentText);
      }
    });
  }

  @override
  void initState() {
    super.initState();
    hasText = controllerHasText;
    controller!.addListener(onValueChanged);
    textEditingController!.addListener(onTextChanged);
  }

  void updateDecoration(
      {InputDecoration? decoration,
      List<TextInputFormatter>? inputFormatters,
      TextCapitalization? textCapitalization,
      TextStyle? style,
      TextInputType? keyboardType,
      TextInputAction? textInputAction,
      bool refresh = true}) {
    if (decoration != null) {
      this.decoration = decoration;
    }

    if (inputFormatters != null) {
      this.inputFormatters = inputFormatters;
    }

    if (textCapitalization != null) {
      this.textCapitalization = textCapitalization;
    }

    if (style != null) {
      this.style = style;
    }

    if (keyboardType != null) {
      this.keyboardType = keyboardType;
    }

    if (textInputAction != null) {
      this.textInputAction = textInputAction;
    }

    this.decoration = this.decoration.copyWith(
          errorText: textEditingController!.error,
          suffixIcon: IconButton(
            icon: getSuffixIcon,
//        iconSize: this.customButtonIconSize,
//        padding: EdgeInsets.only(right: 0),
            onPressed: !enabled
                ? null
                : () {
                    if (!hasText && !showingSuggestions) {
                      showAllSuggestions();
                    } else {
                      if (showingSuggestions) hideAllSuggestions();
                      clear();
                      setState(() {});
                    }
                  },
          ),
          suffixText: null,
        );

    updatedTextField();

    if (refresh) setState(() {});
  }

  Icon get getSuffixIcon {
    if (!hasText && !showingSuggestions) {
      return const Icon(Icons.arrow_drop_down);
    }
    return const Icon(Icons.clear);
  }

  void onTextChanged() {
    if (currentText == textEditingController!.text) return;
    currentText = textEditingController!.text;
    updateOverlay(currentText);

    if (textChanged != null) {
      textChanged!(currentText);
    }

    if (controllerHasText != hasText) {
      hasText = controllerHasText;
//      setState(() {});
    }
  }

  void onValueChanged() {
    if (controller?.value?.text != null) {
      textEditingController?.text = controller?.value?.text as String;
    }
    hideAllSuggestions();
  }

  void updatedTextField() {
    textField = TextField(
      enabled: enabled,
      inputFormatters: inputFormatters,
      textCapitalization: textCapitalization,
      decoration: decoration,
      style: style,
      keyboardType: keyboardType,
      focusNode: focusNode ?? FocusNode(),
      controller: textEditingController ?? AdeptTextEditingController(),
      textInputAction: textInputAction,
//      onChanged: (newText){}, //onChanged function is managed above in the onTextChanged function
      onTap: () {
        updateOverlay(currentText);
      },
      onSubmitted: (submittedText) =>
          triggerSubmitted(submittedText: submittedText),
    );
  }

  bool get controllerHasText => textEditingController!.isNotEmpty;

  void triggerSubmitted({submittedText}) {
    submittedText == null
        ? textSubmitted!(currentText)
        : textSubmitted!(submittedText);

    if (clearOnSubmit) {
      clear();
    }
  }

  void clear() {
    textEditingController!.clear();
    currentText = "";
    updateOverlay();
    valueChanged!(null);
    showingSuggestions = false;
  }

  void addSuggestion(AutoCompleteItem<T> autoCompleteItem) {
    autoCompleteItems.add(autoCompleteItem);
    updateOverlay(currentText);
  }

  void removeSuggestion(AutoCompleteItem<T> autoCompleteItem) {
    autoCompleteItems.contains(autoCompleteItem)
        ? autoCompleteItems.remove(autoCompleteItem)
        : throw "List does not contain autoCompleteItem and therefore cannot be removed";
    updateOverlay(currentText);
  }

  void updateSuggestions(List<AutoCompleteItem<T>> autoCompleteItems) {
    this.autoCompleteItems = autoCompleteItems;
    updateOverlay(currentText);
  }

  double? get autoCompleteItemsBoxSize {
    if (filteredAutoCompleteItems == null ||
        filteredAutoCompleteItems!.isEmpty) {
      return null;
    }
    double items = double.tryParse("${filteredAutoCompleteItems!.length}")!;
    double estimatedSize = items * autoCompleteItemItemHeight;
    return estimatedSize > autoCompleteItemsBoxMaxHeight
        ? autoCompleteItemsBoxMaxHeight
        : estimatedSize;
  }

  void updateOverlay([String? query]) {
    if (listAutoCompleteItemsEntry == null) {
      final Size textFieldSize = (context.findRenderObject() as RenderBox).size;
      final width = textFieldSize.width;
      final height = textFieldSize.height;
      listAutoCompleteItemsEntry = OverlayEntry(builder: (context) {
        return Positioned(
            width: width,
            child: CompositedTransformFollower(
                link: _layerLink,
                showWhenUnlinked: false,
                offset: Offset(0.0, height),
                child: SizedBox(
                    width: width,
                    height: autoCompleteItemsBoxSize,
                    child: Card(
                        elevation: 8,
                        child: Scrollbar(
                            child: SingleChildScrollView(
                          child: ListView.builder(
                              shrinkWrap: true,
                              primary: false,
                              padding: EdgeInsets.zero,
                              itemCount: filteredAutoCompleteItems!.length,
                              itemBuilder: (BuildContext context, int index) {
                                AutoCompleteItem<T> autoCompleteItem =
                                    filteredAutoCompleteItems![index];
                                return InkWell(
                                    child: Container(
                                      padding: const EdgeInsets.only(
                                          left: 16, right: 16),
                                      child: itemBuilder!(
                                          context, autoCompleteItem),
                                    ),
                                    onTap: () {
                                      setState(() {
                                        if (submitOnAutoCompleteItemTap) {
                                          String newText =
                                              autoCompleteItem.toString();
                                          textField!.controller!.text = newText;
                                          if (!TextUtils.isEmpty(newText)) {
                                            textEditingController!.selection =
                                                TextSelection.fromPosition(
                                                    TextPosition(
                                                        offset:
                                                            newText.length));
                                          }
                                          textField!.focusNode!.unfocus();
                                          if (valueChanged != null) {
                                            valueChanged!(autoCompleteItem);
                                          }
                                          if (clearOnSubmit) {
                                            clear();
                                          }
                                        } else {
                                          String newText =
                                              autoCompleteItem.toString();
                                          textField!.controller!.text = newText;
                                          if (textChanged != null) {
                                            textChanged!(newText);
                                          }
                                        }
                                        hideAllSuggestions();
                                      });
                                    });
                              }),
//                            Column(
//                              children: filteredSuggestions.map((autoCompleteItem) {
//                                return new Row(children: [
//                                  new Expanded(
//                                      child: new InkWell(
//                                          child: itemBuilder(context, autoCompleteItem),
//                                          onTap: () {
//                                            setState(() {
//                                              if (submitOnSuggestionTap) {
//                                                String newText = autoCompleteItem
//                                                    .toString();
//                                                textField.textEditingController.text = newText;
//                                                textField.focusNode.unfocus();
//                                                if(valueChanged != null)
//                                                  valueChanged(autoCompleteItem);
//                                                hideAllSuggestions();
//                                                if (clearOnSubmit) {
//                                                  clear();
//                                                }
//                                              } else {
//                                                String newText = autoCompleteItem
//                                                    .toString();
//                                                textField.textEditingController.text = newText;
//                                                if(textChanged != null)
//                                                  textChanged(newText);
//                                              }
//                                              hideAllSuggestions();
//                                            });
//                                          }))
//                                ]);
//                              }).toList(),
//                            ),
                        ))))));
      });
      Overlay.of(context).insert(listAutoCompleteItemsEntry!);
    }

    filteredAutoCompleteItems = getSuggestions(autoCompleteItems, itemSorter,
        itemFilter, autoCompleteItemsAmount, query);

    listAutoCompleteItemsEntry!.markNeedsBuild();
  }

  void showAllSuggestions() {
    if (listAutoCompleteItemsEntry == null) updateOverlay();
    Future(() {
      filteredAutoCompleteItems = autoCompleteItems;
      listAutoCompleteItemsEntry!.markNeedsBuild();
      showingSuggestions = true;
      textField!.focusNode!.unfocus();
      setState(() {});
    });
  }

  void hideAllSuggestions() {
    Future(() {
      filteredAutoCompleteItems = [];
      listAutoCompleteItemsEntry!.markNeedsBuild();
      showingSuggestions = false;
      setState(() {});
    });
  }

  void toggleSuggestions() {
    if (filteredAutoCompleteItems == null ||
        filteredAutoCompleteItems!.isEmpty) {
      showAllSuggestions();
    } else {
      hideAllSuggestions();
    }
  }

  List<AutoCompleteItem<T>> getSuggestions(
      List<AutoCompleteItem<T>> autoCompleteItems,
      Comparator<AutoCompleteItem<T>>? sorter,
      Filter<T>? filter,
      int maxAmount,
      String? query) {
    if (null == query || query.length < minLength) {
      return [];
    }

    autoCompleteItems =
        autoCompleteItems.where((item) => filter!(item, query)).toList();
    autoCompleteItems.sort(sorter);
    if (autoCompleteItems.length > maxAmount) {
      autoCompleteItems = autoCompleteItems.sublist(0, maxAmount);
    }
    return autoCompleteItems;
  }

  @override
  void dispose() {
    controller!.removeListener(onValueChanged);
    textEditingController!.removeListener(onTextChanged);
    // if we created our own focus node and textEditingController, dispose of them
    // otherwise, let the caller dispose of their own instances
    if (focusNode == null) {
      textField!.focusNode!.dispose();
    }
    if (textEditingController == null) {
      textField!.controller!.dispose();
    }
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    updateDecoration(refresh: false);
    return CompositedTransformTarget(
      link: _layerLink,
      child: Theme(
        data: Theme.of(context).copyWith(splashColor: Colors.transparent),
        child: textField!,
      ),
    );
  }
}
