import 'package:flutter/material.dart';
import 'package:adeptutils/adeptutils.dart';

// ignore_for_file: must_be_immutable
class ListPreference extends StatefulWidget {
  final String title;
  final String? summary;
  final String? dialogMessage;
  final String localKey;
  final String? defaultEntry;
  final String? defaultEntryValue;
  final Widget? leading;
  final Widget? trailing;
  final double listHeight;
  final bool enabled;
  final bool useEntryAsSummary;
  final bool barrierDismissible;
  final bool resetOnException;
  final String cancelText;
  final List<String>? entries;
  final List<String>? entryValues;
  final Function(dynamic value)? onValidate;
  final Function(dynamic value)? onSave;
  double? textScaleFactor;

  ListPreference(
    this.title,
    this.localKey, {
    this.summary,
    this.dialogMessage,
    this.defaultEntry,
    this.defaultEntryValue,
    this.leading,
    this.trailing,
    this.listHeight = 220,
    this.enabled = true,
    this.useEntryAsSummary = false,
    this.barrierDismissible = true,
    this.resetOnException = true,
    this.cancelText = "CANCEL",
    this.entries,
    this.entryValues,
    this.onValidate,
    this.onSave,
    this.textScaleFactor,
  });

  @override
  ListPreferenceState createState() => ListPreferenceState();
}

class ListPreferenceState extends State<ListPreference> {
  String? summary;

  @override
  void initState() {
    super.initState();
    String? entry = SharedPreferencesUtils.getString("${widget.localKey}_entry",
        defaultValue: widget.defaultEntry);
    summary = widget.useEntryAsSummary && !TextUtils.isEmpty(entry)
        ? entry
        : widget.summary;
  }

  @override
  Widget build(BuildContext context) {
    return ListTile(
      title: Text(widget.title),
      subtitle: summary != null ? Text(summary!) : null,
      leading: widget.leading,
      trailing: widget.trailing,
      enabled: widget.enabled,
      onTap: () {
        showDialog(
          context: context,
          barrierDismissible: widget.barrierDismissible,
          builder: (context) => _CustomDialog(this),
        );
      },
    );
  }
}

class _CustomDialog extends StatefulWidget {
  final ListPreferenceState preference;

  const _CustomDialog(this.preference);

  @override
  _CustomDialogState createState() => _CustomDialogState();
}

class _CustomDialogState extends State<_CustomDialog> {
  String? entryValue;
  String? entry;
  String? error;

  @override
  void initState() {
    super.initState();
    error = null;
    entryValue = SharedPreferencesUtils.getString(
        widget.preference.widget.localKey,
        defaultValue: widget.preference.widget.defaultEntryValue);
    entry = SharedPreferencesUtils.getString(
        "${widget.preference.widget.localKey}_entry",
        defaultValue: widget.preference.widget.defaultEntry);
  }

  @override
  Widget build(BuildContext context) {
    final dialog = AlertDialog(
      contentPadding: EdgeInsets.zero,
      titlePadding: const EdgeInsets.all(24),
      title: Text(widget.preference.widget.title),
      content: Column(
        mainAxisSize: MainAxisSize.min,
        children: <Widget>[
          !TextUtils.isEmpty(widget.preference.widget.dialogMessage)
              ? Text(widget.preference.widget.dialogMessage!,
                  style: TextStyle(
                      color: Theme.of(context).textTheme.displaySmall!.color))
              : const SizedBox(),
          SizedBox(
            height: !TextUtils.isEmpty(widget.preference.widget.dialogMessage)
                ? 16
                : 0,
          ),
          buildRadioGroup(context),
          !TextUtils.isEmpty(error)
              ? Padding(
                  padding: const EdgeInsets.all(16),
                  child: Text(
                    error!,
                    textAlign: TextAlign.center,
                    style: TextStyle(color: Colors.red.shade900),
                  ),
                )
              : const SizedBox(),
        ],
      ),
      actions: <Widget>[
        TextButton(
          child: Text(widget.preference.widget.cancelText),
          onPressed: () {
            if (error != null) {
              error = null;
              entryValue = SharedPreferencesUtils.getString(
                  widget.preference.widget.localKey,
                  defaultValue: widget.preference.widget.defaultEntryValue);
              entry = SharedPreferencesUtils.getString(
                  "${widget.preference.widget.localKey}_entry",
                  defaultValue: widget.preference.widget.defaultEntry);
              widget.preference.summary =
                  widget.preference.widget.useEntryAsSummary &&
                          !TextUtils.isEmpty(entry)
                      ? entry
                      : widget.preference.widget.summary;
            }
            setState(() {});
            Navigator.of(context).pop();
          },
        ),
      ],
    );
    return widget.preference.widget.textScaleFactor != null
        ? MediaQuery(
            data: MediaQuery.of(context).copyWith(
                textScaleFactor: widget.preference.widget.textScaleFactor),
            child: dialog,
          )
        : dialog;
  }

  Widget buildRadioGroup(context) {
    if (widget.preference.widget.entries == null ||
        widget.preference.widget.entryValues == null ||
        widget.preference.widget.entries!.length !=
            widget.preference.widget.entryValues!.length) {
      return const SizedBox();
    }
    return SizedBox(
      width: MediaQuery.of(context).size.width,
      height: widget.preference.widget.listHeight,
      child: ListView.builder(
        primary: false,
        physics: const BouncingScrollPhysics(),
        padding: EdgeInsets.zero,
        itemCount: widget.preference.widget.entries!.length,
        itemBuilder: (BuildContext context, int index) {
          return RadioListTile<String>(
            title: Text(widget.preference.widget.entries![index]),
            value: widget.preference.widget.entryValues![index],
            groupValue: entryValue,
            onChanged: (value) {
              try {
                error = null;
                if (widget.preference.widget.onValidate != null) {
                  widget.preference.widget.onValidate!(value);
                }
                SharedPreferencesUtils.putString(
                    widget.preference.widget.localKey, (entryValue = value)!);
                SharedPreferencesUtils.putString(
                    "${widget.preference.widget.localKey}_entry",
                    entry = widget.preference.widget.entries![index]);
                if (widget.preference.widget.useEntryAsSummary) {
                  widget.preference.summary = entry;
                }
                widget.preference.setState(() {});
                Navigator.of(context).pop();
                if (widget.preference.widget.onSave != null) {
                  widget.preference.widget.onSave!(value);
                }
              } catch (e) {
                error = ExceptionUtils.parseException(e).toString();
                entryValue = value;
                entry = widget.preference.widget.entries![index];
                setState(() {});
              }
            },
          );
        },
      ),
    );
  }
}
