import 'package:flutter/material.dart';
import 'package:adeptutils/adeptutils.dart';
import 'dart:convert';

// ignore_for_file: must_be_immutable
class MultiSelectListPreference extends StatefulWidget {
  final String title;
  final String? summary;
  final String? dialogMessage;
  final String localKey;
  final List<String>? defaultEntries;
  final List<String>? defaultEntryValues;
  final Widget? leading;
  final Widget? trailing;
  final double listHeight;
  final bool enabled;
  final bool barrierDismissible;
  final bool resetOnException;
  final String acceptText;
  final String cancelText;
  final List<String>? entries;
  final List<String>? entryValues;
  final Function(List<String>? value)? onValidate;
  final Function(List<String>? value)? onSave;
  double? textScaleFactor;

  MultiSelectListPreference(
    this.title,
    this.localKey, {
    this.summary,
    this.dialogMessage,
    this.defaultEntries,
    this.defaultEntryValues,
    this.leading,
    this.trailing,
    this.listHeight = 220,
    this.enabled = true,
    this.barrierDismissible = true,
    this.resetOnException = true,
    this.acceptText = "ACCEPT",
    this.cancelText = "CANCEL",
    this.entries,
    this.entryValues,
    this.onValidate,
    this.onSave,
    this.textScaleFactor,
  });

  @override
  MultiSelectListPreferenceState createState() =>
      MultiSelectListPreferenceState();
}

class MultiSelectListPreferenceState extends State<MultiSelectListPreference> {
  String? summary;

  @override
  void initState() {
    super.initState();
    summary = widget.summary;
  }

  @override
  Widget build(BuildContext context) {
    return ListTile(
      title: Text(widget.title),
      subtitle: summary != null ? Text(summary!) : null,
      leading: widget.leading,
      trailing: widget.trailing,
      enabled: widget.enabled,
      onTap: () {
        showDialog(
          context: context,
          barrierDismissible: widget.barrierDismissible,
          builder: (context) => _CustomDialog(this),
        );
      },
    );
  }
}

class _CustomDialog extends StatefulWidget {
  final MultiSelectListPreferenceState preference;

  const _CustomDialog(this.preference);

  @override
  _CustomDialogState createState() => _CustomDialogState();
}

class _CustomDialogState extends State<_CustomDialog> {
  Map<String, bool> map = {};
  List<String>? selectedValues;
  String? error;

  @override
  void initState() {
    super.initState();
    resetData();
  }

  @override
  Widget build(BuildContext context) {
    final dialog = AlertDialog(
      contentPadding: EdgeInsets.zero,
      titlePadding: const EdgeInsets.all(24),
      title: Text(widget.preference.widget.title),
      content: Column(
        mainAxisSize: MainAxisSize.min,
        children: <Widget>[
          !TextUtils.isEmpty(widget.preference.widget.dialogMessage)
              ? Text(widget.preference.widget.dialogMessage!,
                  style: TextStyle(
                      color: Theme.of(context).textTheme.displaySmall!.color))
              : const Divider(
                  color: Colors.transparent,
                  height: 0,
                ),
          Divider(
            color: Colors.transparent,
            height: !TextUtils.isEmpty(widget.preference.widget.dialogMessage)
                ? 16
                : 0,
          ),
          buildRadioGroup(context),
          !TextUtils.isEmpty(error)
              ? Padding(
                  padding: const EdgeInsets.all(16),
                  child: Text(
                    error!,
                    textAlign: TextAlign.center,
                    style: TextStyle(color: Colors.red.shade900),
                  ),
                )
              : const Divider(
                  color: Colors.transparent,
                  height: 0,
                ),
        ],
      ),
      actions: <Widget>[
        TextButton(
          child: Text(widget.preference.widget.cancelText),
          onPressed: () {
            Navigator.of(context).pop();
          },
        ),
        TextButton(
          child: Text(widget.preference.widget.acceptText),
          onPressed: () {
            try {
              if (error != null) {
                return;
              }
              if (widget.preference.widget.onValidate != null) {
                widget.preference.widget.onValidate!(selectedValues);
              }
              saveData();
              if (widget.preference.widget.onSave != null) {
                widget.preference.widget.onSave!(selectedValues);
              }
              widget.preference.setState(() {});
              Navigator.of(context).pop();
            } catch (e) {
              error = ExceptionUtils.parseException(e).toString();
              setState(() {});
            }
          },
        ),
      ],
    );
    return widget.preference.widget.textScaleFactor != null
        ? MediaQuery(
            data: MediaQuery.of(context).copyWith(
                textScaleFactor: widget.preference.widget.textScaleFactor),
            child: dialog,
          )
        : dialog;
  }

  List<String> tryJsonDecode(String? json) {
    List<dynamic>? values = json != null
        ? jsonDecode(json)
        : widget.preference.widget.defaultEntryValues;
    List<String> listString = [];
    if (values != null && values.isNotEmpty) {
      for (var value in values) {
        listString.add(value);
      }
    }
    return listString;
  }

  void resetData() {
    error = null;
    selectedValues = tryJsonDecode(
        SharedPreferencesUtils.getString(widget.preference.widget.localKey));
    map = {};
    if (selectedValues != null && selectedValues!.isNotEmpty) {
      for (var value in selectedValues!) {
        map[value] = true;
      }
    }
    widget.preference.summary = widget.preference.summary;
  }

  void saveData() {
    selectedValues = [];
    map.forEach((key, value) {
      if (value) selectedValues!.add(key);
    });
    String json = jsonEncode(selectedValues);
    SharedPreferencesUtils.putString(widget.preference.widget.localKey, json);
  }

  bool isValueSelected(int index) {
    bool? value = map[widget.preference.widget.entryValues![index]];
    return value != null && value;
  }

  Widget buildRadioGroup(context) {
    if (widget.preference.widget.entries == null ||
        widget.preference.widget.entryValues == null ||
        widget.preference.widget.entries!.length !=
            widget.preference.widget.entryValues!.length) {
      return const Divider(
        color: Colors.transparent,
        height: 0,
      );
    }
    return Container(
      width: MediaQuery.of(context).size.width,
      padding: const EdgeInsets.only(left: 8, right: 8),
      height: widget.preference.widget.listHeight,
      child: ListView.builder(
        primary: false,
        padding: EdgeInsets.zero,
        itemCount: widget.preference.widget.entries!.length,
        physics: const BouncingScrollPhysics(),
        itemBuilder: (BuildContext context, int index) {
          return ListTile(
            title: Text(widget.preference.widget.entries![index]),
            leading: isValueSelected(index)
                ? Icon(
                    Icons.check_box,
                    color: Theme.of(context).colorScheme.secondary,
                  )
                : const Icon(
                    Icons.check_box_outline_blank,
                  ),
            onTap: () {
              try {
                error = null;
                bool value = !isValueSelected(index);
                map[widget.preference.widget.entryValues![index]] = value;
                setState(() {});
              } catch (e) {
                error = ExceptionUtils.parseException(e).toString();
                setState(() {});
              }
            },
          );
        },
      ),
    );
  }

//  Widget buildRadioGroup(context){
//    if(widget.preference.widget.entries == null || widget.preference.widget.entryValues == null ||
//        widget.preference.widget.entries.length != widget.preference.widget.entryValues.length)
//      return Divider(color: Colors.transparent, height: 0,);
//    return Container(
//      width: MediaQuery.of(context).size.width,
//      height: widget.preference.widget.listHeight,
//      child: ListView.builder(
//        primary: false,
//        padding: EdgeInsets.zero,
//        itemCount: widget.preference.widget.entries.length,
//        itemBuilder: (BuildContext context, int index) {
//          return CheckboxListTile(
//            title: Text(widget.preference.widget.entries[index]),
//            value: isValueSelected(index),
//            onChanged: (value) {
//              try{
//                error = null;
//                map[widget.preference.widget.entryValues[index]] = value;
//                setState(() {});
//              } catch(e){
//                error = ExceptionUtils.parseException(e).toString();
////                this.entryValue = value;
////                this.entry = widget.preference.widget.entries[index];
//                setState(() {});
//              }
//            },
//          );
//        },
//      ),
//    );
//  }
}
