import 'package:adeptviews/src/preference/preference_service.dart';
import 'package:flutter/material.dart';
import 'package:adeptutils/adeptutils.dart';

// ignore_for_file: must_be_immutable
class TextFieldPreference extends StatefulWidget {
  final String title;
  final String? summary;
  final String? summaryPrefix;
  final String? summarySuffix;
  final TextInputType textInputType;
  final String? dialogMessage;
  final String localKey;
  final String? defaultValue;
  final int? maxLength;
  final String? labelText;
  final String? hintText;
  final Widget? leading;
  final Widget? trailing;
  final bool enabled;
  final bool useValueAsSummary;
  final bool useValueAsSummaryPrefix;
  final bool useValueAsSummarySuffix;
  final bool barrierDismissible;
  final bool resetOnException;
  final String acceptText;
  final String cancelText;
  final Function(AdeptTextEditingController controller)? onValidate;
  final Function(String value)? onSave;
  double? textScaleFactor;

  TextFieldPreference(
    this.title,
    this.localKey, {
    this.summary,
    this.summaryPrefix,
    this.summarySuffix,
    this.dialogMessage,
    this.textInputType = TextInputType.text,
    this.defaultValue,
    this.maxLength,
    this.labelText,
    this.hintText,
    this.leading,
    this.trailing,
    this.enabled = true,
    this.useValueAsSummary = false,
    this.useValueAsSummaryPrefix = false,
    this.useValueAsSummarySuffix = false,
    this.barrierDismissible = true,
    this.resetOnException = true,
    this.acceptText = "ACCEPT",
    this.cancelText = "CANCEL",
    this.onValidate,
    this.onSave,
    this.textScaleFactor,
  });

  @override
  TextFieldPreferenceState createState() => TextFieldPreferenceState();
}

class TextFieldPreferenceState extends State<TextFieldPreference> {
  String? summary;
  String? value;

  @override
  void initState() {
    super.initState();
    value = PrefService.getString(
          widget.localKey,
        ) ??
        widget.defaultValue;
    updateSummary();
  }

  void updateSummary() {
    summary = widget.summary;
    summary ??= "";
    summary = (widget.summaryPrefix ?? "") +
        (widget.useValueAsSummary && !TextUtils.isEmpty(value)
            ? value!
            : widget.summary!) +
        (widget.summarySuffix != null ? widget.summarySuffix! : "");
    if (widget.useValueAsSummary && !TextUtils.isEmpty(value)) {
      summary = (widget.summaryPrefix ?? "") +
          value! +
          (widget.summarySuffix != null ? widget.summarySuffix! : "");
    } else if (widget.useValueAsSummaryPrefix && !TextUtils.isEmpty(value)) {
      summary = "(${widget.summaryPrefix ?? ""}$value) $summary";
    } else if (widget.useValueAsSummarySuffix && !TextUtils.isEmpty(value)) {
      summary = "$summary (${widget.summarySuffix ?? ""}$value)";
    }
  }

  @override
  Widget build(BuildContext context) {
    return ListTile(
      title: Text(widget.title),
      subtitle: summary != null ? Text(summary!) : null,
      leading: widget.leading,
      trailing: widget.trailing,
      enabled: widget.enabled,
      onTap: () {
        showDialog(
          context: context,
          barrierDismissible: widget.barrierDismissible,
          builder: (context) => _CustomDialog(this),
        );
      },
    );
  }
}

class _CustomDialog extends StatefulWidget {
  final TextFieldPreferenceState preference;

  const _CustomDialog(this.preference);

  @override
  _CustomDialogState createState() => _CustomDialogState();
}

class _CustomDialogState extends State<_CustomDialog> {
  final controller = AdeptTextEditingController();
  String? error;

  @override
  void initState() {
    super.initState();
    controller.text = widget.preference.value?.toString() ?? '';
  }

  @override
  Widget build(BuildContext context) {
    final dialog = AlertDialog(
      title: Text(widget.preference.widget.title),
      content: SingleChildScrollView(
        child: Column(
          children: <Widget>[
            if (!TextUtils.isEmpty(widget.preference.widget.dialogMessage))
              Padding(
                padding: const EdgeInsets.only(bottom: 16),
                child: Text(widget.preference.widget.dialogMessage!,
                    style: TextStyle(
                        color:
                            Theme.of(context).textTheme.displaySmall!.color)),
              ),
            TextField(
              keyboardType: widget.preference.widget.textInputType,
              maxLength: widget.preference.widget.maxLength,
              controller: controller,
              decoration: InputDecoration(
                labelText: widget.preference.widget.labelText,
                hintText: widget.preference.widget.hintText,
                errorText: controller.error,
              ),
            ),
          ],
        ),
      ),
      actions: <Widget>[
        TextButton(
          child: Text(widget.preference.widget.cancelText),
          onPressed: () {
            controller.text = widget.preference.value?.toString() ?? '';
            controller.clearError();
            setState(() {});
            Navigator.of(context).pop();
          },
        ),
        TextButton(
          child: Text(widget.preference.widget.acceptText),
          onPressed: () {
            try {
              controller.clearError();
              if (widget.preference.widget.onValidate != null) {
                widget.preference.widget.onValidate!(controller);
              }
              PrefService.setString(widget.preference.widget.localKey,
                  widget.preference.value = controller.text);
              widget.preference.updateSummary();
              widget.preference.setState(() {});
              Navigator.of(context).pop();
              if (widget.preference.widget.onSave != null) {
                widget.preference.widget.onSave!(controller.text);
              }
            } catch (e) {
              controller.error = ExceptionUtils.parseException(e).toString();
              setState(() {});
            }
          },
        ),
      ],
    );
    return widget.preference.widget.textScaleFactor != null
        ? MediaQuery(
            data: MediaQuery.of(context).copyWith(
                textScaleFactor: widget.preference.widget.textScaleFactor),
            child: dialog,
          )
        : dialog;
  }
}
