import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:adeptutils/adeptutils.dart';
import 'dart:async';

enum DateTimePickerMode {
  DATE,
  TIME,
  BOTH,
}

typedef OnDateTimeSelected = Function(DateTime? dateTime);

// ignore_for_file: must_be_immutable, annotate_overrides, overridden_fields
class DateTextField extends StatefulWidget {
  static const String DATE_FORMAT = "EEE, d MMM yyyy hh:mm a";

  Key? key;
  AdeptValueController<DateTime>? controller;
  FocusNode? focusNode;
  InputDecoration decoration;
  TextInputType? keyboardType;
  TextInputAction? textInputAction;
  TextCapitalization textCapitalization;
  TextStyle? style;
  TextAlign textAlign;
  TextDirection? textDirection;
  bool autofocus;
  bool obscureText;
  bool autocorrect;
  int maxLines;
  int? maxLength;
  bool maxLengthEnforced;
  ValueChanged<String>? onChanged;
  VoidCallback? onEditingComplete;
  ValueChanged<String>? onSubmitted;
  List<TextInputFormatter>? inputFormatters;
  bool? enabled;
  double cursorWidth;
  Radius? cursorRadius;
  Color? cursorColor;
  Brightness? keyboardAppearance;
  EdgeInsets scrollPadding;
  bool enableInteractiveSelection;
  GestureTapCallback? onTap;
  VoidCallback? clearButtonAction;
  Color? clearButtonColor;
  IconData? clearButtonIcon;
  bool autoUpdateDecoration;

  DateTimePickerMode dateTimePickerMode;
  String? dateFormat;
  DateTime? defaultDate;
  OnDateTimeSelected? onDateTimeSelected;

  TextEditingController? textController;

  DateTextField({
    this.key,
    this.controller,
    this.focusNode,
    this.decoration = const InputDecoration(),
    this.keyboardType,
    this.textInputAction,
    this.textCapitalization = TextCapitalization.none,
    this.style,
    this.textAlign = TextAlign.start,
    this.textDirection,
    this.autofocus = false,
    this.obscureText = false,
    this.autocorrect = true,
    this.maxLines = 1,
    this.maxLength,
    this.maxLengthEnforced = true,
    this.onChanged,
    this.onEditingComplete,
    this.onSubmitted,
    this.inputFormatters,
    this.enabled,
    this.cursorWidth = 2.0,
    this.cursorRadius,
    this.cursorColor,
    this.keyboardAppearance,
    this.scrollPadding = const EdgeInsets.all(20.0),
    this.enableInteractiveSelection = true,
    this.onTap,
    this.clearButtonAction,
    this.clearButtonColor,
    this.clearButtonIcon,
    this.autoUpdateDecoration = true,
    this.dateTimePickerMode = DateTimePickerMode.BOTH,
    this.dateFormat,
    this.defaultDate,
    this.onDateTimeSelected,
  }) {
    enabled ??= true;
    dateFormat ??= DATE_FORMAT;
    textController ??= TextEditingController();
    controller ??= AdeptValueController(value: defaultDate);
    if (controller!.value != null) {
      textController!.text = AdeptDateTime(dateAsDateTime: controller!.value)
          .format(format: dateFormat);
    }
  }

  @override
  DateTextFieldState createState() => DateTextFieldState();
}

class DateTextFieldState extends State<DateTextField> {
  bool hasText = false;
  bool hasError = false;
  Color? onTextColor, onClearTextColor;

  DateTextFieldState();

  @override
  void initState() {
    super.initState();
    hasText = controllerHasText;
    hasError = widget.controller!.hasError;
    widget.controller!.addListener(onDateChanged);
    widget.textController!.addListener(onTextChanged);
    if (widget.focusNode != null) {
      widget.focusNode!.addListener(() {
        if (widget.focusNode!.hasFocus && widget.enabled!) showPicker();
      });
    }
  }

  void updateDecoration(context) {
    if (!widget.autoUpdateDecoration) return;
    onTextColor ??= Theme.of(context).primaryColor;
    onClearTextColor ??= Theme.of(context).disabledColor;

    widget.decoration = widget.decoration.copyWith(
      errorText: widget.controller!.error,
      errorStyle: widget.decoration.errorStyle != null
          ? widget.decoration.errorStyle!
              .copyWith(color: Theme.of(context).colorScheme.error)
          : TextStyle(color: Theme.of(context).colorScheme.error),
      labelStyle: widget.decoration.labelStyle != null
          ? widget.decoration.labelStyle!
              .copyWith(color: hasText ? onTextColor : onClearTextColor)
          : TextStyle(color: hasText ? onTextColor : onClearTextColor),
      disabledBorder: widget.decoration.disabledBorder != null
          ? widget.decoration.disabledBorder!.copyWith(
              borderSide: BorderSide(
              color: hasText ? onTextColor! : onClearTextColor!,
              width: hasText ? 2 : 1,
            ))
          : UnderlineInputBorder(
              borderSide: BorderSide(
              color: hasText ? onTextColor! : onClearTextColor!,
              width: hasText ? 2 : 1,
            )),
    );
  }

  bool get controllerHasText => widget.textController?.text.isNotEmpty ?? false;

  void onDateChanged() {
    DateTime? dateTime = widget.controller!.value;
    if (dateTime != null) {
      widget.textController!.text = AdeptDateTime(dateAsDateTime: dateTime)
          .format(format: widget.dateFormat);
    }
    if (hasError != widget.controller!.hasError) {
      hasError = widget.controller!.hasError;
      setState(() {});
    }
  }

  void notifyOnDateTimeSelected(DateTime? dateTime) {
    widget.controller!.value = dateTime;
    if (widget.onDateTimeSelected != null) widget.onDateTimeSelected!(dateTime);
  }

  void onTextChanged() {
    bool controllerHasText = this.controllerHasText;
    if (controllerHasText != hasText) {
      hasText = controllerHasText;
      hasError = widget.controller!.hasError;
      if (!hasText) notifyOnDateTimeSelected(null);
      setState(() {});
    }
  }

  Icon getCustomButtonIcon(BuildContext context) {
    widget.clearButtonColor ??= Theme.of(context).primaryColor;
    if (widget.clearButtonIcon != null) {
      return Icon(
        widget.clearButtonIcon,
        color: widget.clearButtonColor,
      );
    }
    return Icon(
      Icons.clear,
      color: widget.clearButtonColor,
    );
  }

  Future<DateTime?> get _showDatePicker async => await showDatePicker(
        context: context,
        initialDate: widget.controller!.value != null
            ? widget.controller!.value!
            : DateTime.now(),
        firstDate: DateTime(1900),
        lastDate: DateTime(2100),
      );

  Future<TimeOfDay?> get _showTimePicker async => await showTimePicker(
        context: context,
        initialTime: widget.controller!.value != null
            ? TimeOfDay(
                hour: widget.controller!.value!.hour,
                minute: widget.controller!.value!.minute)
            : TimeOfDay.now(),
      );

  void showPicker() async {
    DateTime? dateTime;
    switch (widget.dateTimePickerMode) {
      case DateTimePickerMode.BOTH:
        dateTime = await _showDatePicker;
        if (dateTime == null) return;
        TimeOfDay? time = await _showTimePicker;
        if (time == null) return;
        dateTime = DateTime(
          dateTime.year,
          dateTime.month,
          dateTime.day,
          time.hour,
          time.minute,
          DateTime.now().second,
        );
        break;
      case DateTimePickerMode.DATE:
        dateTime = await _showDatePicker;
        if (dateTime == null) return;
        dateTime = DateTime(
          dateTime.year,
          dateTime.month,
          dateTime.day,
        );
        break;
      case DateTimePickerMode.TIME:
        TimeOfDay? time = await _showTimePicker;
        if (time == null) return;
        dateTime = DateTime.now();
        dateTime = DateTime(
          dateTime.year,
          dateTime.month,
          dateTime.day,
          time.hour,
          time.minute,
          dateTime.second,
        );
        break;
    }
    notifyOnDateTimeSelected(dateTime);
  }

  @override
  Widget build(BuildContext context) {
    updateDecoration(context);
    return Stack(
      children: <Widget>[
        InkWell(
          onTap: !widget.enabled! ? null : showPicker,
          child: TextField(
            key: widget.key,
            controller: widget.textController,
            focusNode: widget.focusNode,
            decoration: widget.decoration,
            keyboardType: widget.keyboardType,
            textInputAction: widget.textInputAction,
            textCapitalization: widget.textCapitalization,
            style: widget.style,
            textAlign: widget.textAlign,
            textDirection: widget.textDirection,
            autofocus: widget.autofocus,
            obscureText: widget.obscureText,
            autocorrect: widget.autocorrect,
            maxLines: widget.maxLines,
            maxLength: widget.maxLength,
            onChanged: widget.onChanged,
            onEditingComplete: widget.onEditingComplete,
            onSubmitted: widget.onSubmitted,
            inputFormatters: widget.inputFormatters,
            enabled: false,
            cursorWidth: widget.cursorWidth,
            cursorRadius: widget.cursorRadius,
            cursorColor: widget.cursorColor,
            keyboardAppearance: widget.keyboardAppearance,
            scrollPadding: widget.scrollPadding,
            enableInteractiveSelection: widget.enableInteractiveSelection,
            onTap: !widget.enabled! ? null : widget.onTap,
          ),
        ),
        Positioned(
          bottom: 0,
          right: 0,
          child: !widget.enabled! || !hasText
              ? Container()
              : IconButton(
                  icon: getCustomButtonIcon(context),
                  onPressed: () {
                    if (widget.clearButtonAction != null) {
                      widget.clearButtonAction!();
                    } else {
                      widget.textController!.clear();
                    }
                  },
                ),
        ),
      ],
    );
  }

  @override
  void dispose() {
    widget.controller!.removeListener(onDateChanged);
    widget.textController!.removeListener(onTextChanged);
    if (widget.focusNode != null) widget.focusNode!.dispose();
    super.dispose();
  }
}
