import 'dart:async';
import 'package:adeptviews/adeptviews.dart';
import 'package:adeptviews/src/text/timer/digit.dart';
import 'package:flutter/material.dart';
import 'dart:math' as math;

// ignore_for_file: no_logic_in_create_state
class SlideCountDownClock extends StatefulWidget {
  final Duration duration;
  final TextStyle textStyle;
  final TextStyle? separatorTextStyle;
  final String separator;
  final BoxDecoration? decoration;
  final SlideDirection slideDirection;
  final VoidCallback? onDone;
  final EdgeInsets padding;
  final bool tightLabel;
  final bool shouldShowDays;
  final TimerType type;

  const SlideCountDownClock({
    Key? key,
    required this.duration,
    this.textStyle = const TextStyle(
      fontSize: 30,
    ),
    this.separatorTextStyle,
    this.decoration,
    this.tightLabel = false,
    this.separator = "",
    this.slideDirection = SlideDirection.DOWN,
    this.onDone,
    @Deprecated('Use type instead') this.shouldShowDays = false,
    this.type = TimerType.HOURS_MINUTES_SECONDS,
    this.padding = EdgeInsets.zero,
  }) : super(key: key);

  @override
  SlideCountDownClockState createState() =>
      SlideCountDownClockState(duration, shouldShowDays, type);
}

class SlideCountDownClockState extends State<SlideCountDownClock> {
  TimerType type;
  Duration timeLeft;
  StreamSubscription? onDoneStreamSubscription;
  late Stream<DateTime> timeStream;

  SlideCountDownClockState(this.timeLeft, bool? shouldShowDays, this.type) {
    if (shouldShowDays ?? false) type = TimerType.DAYS_HOURS_MINUTES_SECONDS;
  }

  @override
  void initState() {
    super.initState();
    _init();
  }

  void _init() {
    const frequency = Duration(seconds: 1);

    final onDoneStream = Stream<DateTime?>.periodic(frequency, (_) {
      timeLeft -= const Duration(seconds: 1);
      if (timeLeft.inSeconds == 0) {
        onDoneStreamSubscription?.cancel();
        Future.delayed(const Duration(seconds: 1), () {
          widget.onDone?.call();
        });
      }
      return null;
    });
    onDoneStreamSubscription = onDoneStream.listen(null);

    timeStream = Stream<DateTime>.periodic(frequency, (_) => DateTime(0))
        .take(timeLeft.inSeconds)
        .asBroadcastStream();
  }

  @override
  Widget build(BuildContext context) {
    Widget dayDigits;
    if (timeLeft.inDays > 99) {
      List<DigitFunction> digits = [];
      for (int i = timeLeft.inDays.toString().length - 1; i >= 0; i--) {
        digits.add((DateTime? time) =>
            ((timeLeft.inDays) ~/ math.pow(10, i) % math.pow(10, 1)).toInt());
      }
      dayDigits = _buildDigitForLargeNumber(
          timeStream, digits, DateTime.now(), 'daysHundreds');
    } else {
      dayDigits = _buildDigit(
        timeStream,
        (DateTime? time) => (timeLeft.inDays) ~/ 10,
        (DateTime? time) => (timeLeft.inDays) % 10,
        DateTime.now(),
        "Days",
      );
    }

    return Row(
      mainAxisSize: MainAxisSize.min,
      children: <Widget>[
        if (type.showDays) ...[
          dayDigits,
          if (widget.separator.isNotEmpty) _buildSeparator()
        ],
        if (type.showHours) ...[
          _buildDigit(
            timeStream,
            (DateTime? time) => (timeLeft.inHours % 24) ~/ 10,
            (DateTime? time) => (timeLeft.inHours % 24) % 10,
            DateTime.now(),
            "Hours",
          ),
          if (widget.separator.isNotEmpty) _buildSeparator(),
        ],
        if (type.showMinutes) ...[
          _buildDigit(
            timeStream,
            (DateTime? time) => (timeLeft.inMinutes % 60) ~/ 10,
            (DateTime? time) => (timeLeft.inMinutes % 60) % 10,
            DateTime.now(),
            "minutes",
          ),
          if (widget.separator.isNotEmpty) _buildSeparator(),
        ],
        _buildDigit(
          timeStream,
          (DateTime? time) => (timeLeft.inSeconds % 60) ~/ 10,
          (DateTime? time) => (timeLeft.inSeconds % 60) % 10,
          DateTime.now(),
          "seconds",
        )
      ],
    );
  }

  Widget _buildSeparator() {
    return Text(
      widget.separator,
      style: widget.separatorTextStyle ?? widget.textStyle,
    );
  }

  Widget _buildDigitForLargeNumber(
    Stream<DateTime> timeStream,
    List<DigitFunction> digits,
    DateTime startTime,
    String id,
  ) {
    String timeLeftString = timeLeft.inDays.toString();
    List<Widget> rows = [];
    for (int i = 0; i < timeLeftString.toString().length; i++) {
      rows.add(
        Container(
          decoration: widget.decoration,
          padding: widget.tightLabel
              ? const EdgeInsets.only(left: 3)
              : EdgeInsets.zero,
          child: Digit(
            padding: widget.padding,
            itemStream: timeStream.map<int>(digits[i]),
            initValue: digits[i](startTime),
            decoration: widget.decoration,
            slideDirection: widget.slideDirection,
            textStyle: widget.textStyle,
          ),
        ),
      );
    }

    return Column(
      mainAxisSize: MainAxisSize.min,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: <Widget>[
        Row(
          mainAxisSize: MainAxisSize.min,
          mainAxisAlignment: MainAxisAlignment.center,
          children: rows,
        ),
      ],
    );
  }

  Widget _buildDigit(
    Stream<DateTime> timeStream,
    DigitFunction tensDigit,
    DigitFunction onesDigit,
    DateTime startTime,
    String id,
  ) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: <Widget>[
        Row(
          mainAxisSize: MainAxisSize.min,
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            Container(
              decoration: widget.decoration,
              padding: widget.tightLabel
                  ? const EdgeInsets.only(left: 3)
                  : EdgeInsets.zero,
              child: Digit(
                padding: widget.padding,
                itemStream: timeStream.map<int>(tensDigit),
                initValue: tensDigit(startTime),
                decoration: widget.decoration,
                slideDirection: widget.slideDirection,
                textStyle: widget.textStyle,
              ),
            ),
            Container(
              decoration: widget.decoration,
              padding: widget.tightLabel
                  ? const EdgeInsets.only(right: 3)
                  : EdgeInsets.zero,
              child: Digit(
                padding: widget.padding,
                itemStream: timeStream.map<int>(onesDigit),
                initValue: onesDigit(startTime),
                decoration: widget.decoration,
                slideDirection: widget.slideDirection,
                textStyle: widget.textStyle,
              ),
            ),
          ],
        ),
      ],
    );
  }

  @override
  void dispose() {
    onDoneStreamSubscription?.cancel();
    super.dispose();
  }
}
