import 'dart:async';
import 'package:adeptviews/src/text/timer/digit.dart';
import 'package:adeptviews/src/text/timer/slide_direction.dart';
import 'package:adeptviews/src/text/timer/timer_type.dart';
import 'package:flutter/material.dart';
import 'dart:math' as math;

// ignore_for_file: no_logic_in_create_state
class SlideCountForwardClock extends StatefulWidget {
  final DateTime? startTime;
  final Duration? timeCounter;
  final TextStyle textStyle;
  final TextStyle? separatorTextStyle;
  final String separator;
  final BoxDecoration? decoration;
  final EdgeInsets padding;
  final bool tightLabel;
  final bool showDaysAlways;
  final TimerType type;
  final bool paused;

  const SlideCountForwardClock({
    Key? key,
    this.startTime,
    this.timeCounter,
    this.textStyle = const TextStyle(
      fontSize: 30,
    ),
    this.separatorTextStyle,
    this.decoration,
    this.padding = EdgeInsets.zero,
    this.tightLabel = false,
    this.separator = "",
    @Deprecated('Use type instead') this.showDaysAlways = false,
    this.type = TimerType.HOURS_MINUTES_SECONDS,
    this.paused = false,
  }) : super(key: key);

  @override
  SlideCountForwardClockState createState() =>
      SlideCountForwardClockState(startTime, timeCounter, showDaysAlways, type);
}

class SlideCountForwardClockState extends State<SlideCountForwardClock> {
  TimerType type;
  late Duration timeCounter;
  late Stream<DateTime?> timeStream;
  bool isDisposed = false;
  bool decrementFirst = false;
  late SlideDirection slideDirection;

  SlideCountForwardClockState(
      DateTime? startTime, Duration? timeCounter, bool? showDays, this.type) {
    if (timeCounter == null) {
      final now = DateTime.now();
      startTime ??= now;
      int difference =
          now.millisecondsSinceEpoch - startTime.millisecondsSinceEpoch;
      if (difference < 0) decrementFirst = true;
      this.timeCounter = Duration(milliseconds: difference.abs());
    } else {
      this.timeCounter = timeCounter;
    }

    slideDirection = decrementFirst ? SlideDirection.DOWN : SlideDirection.UP;
    if (showDays ?? false) type = TimerType.DAYS_HOURS_MINUTES_SECONDS;
  }

  @override
  void initState() {
    super.initState();
    _init();
  }

  bool get isPaused => widget.paused;

  void _init() {
    final initStream = isPaused
        ? Stream<DateTime?>.value(null)
        : Stream<DateTime?>.periodic(const Duration(milliseconds: 1000), (_) {
            if (decrementFirst) {
              timeCounter -= const Duration(seconds: 1);
              if (timeCounter.inSeconds == 0) {
                decrementFirst = false;
                slideDirection = SlideDirection.UP;
                setState(() {});
              }
            } else {
              timeCounter += const Duration(seconds: 1);
            }
            return null;
          });
    timeStream =
        initStream.takeWhile((element) => !isDisposed).asBroadcastStream();
  }

  @override
  Widget build(BuildContext context) {
    Widget dayDigits;
    if (timeCounter.inDays > 99) {
      List<DigitFunction> digits = [];
      for (int i = timeCounter.inDays.toString().length - 1; i >= 0; i--) {
        digits.add((DateTime? time) =>
            ((timeCounter.inDays) ~/ math.pow(10, i) % math.pow(10, 1))
                .toInt());
      }
      dayDigits = _buildDigitForLargeNumber(
          timeStream, digits, DateTime.now(), 'daysHundreds');
    } else {
      dayDigits = _buildDigit(
        timeStream,
        (DateTime? time) => (timeCounter.inDays) ~/ 10,
        (DateTime? time) => (timeCounter.inDays) % 10,
        DateTime.now(),
        "Days",
      );
    }

    return Row(
      mainAxisSize: MainAxisSize.min,
      children: <Widget>[
        if (type.showDays) ...[
          dayDigits,
          if (widget.separator.isNotEmpty) _buildSeparator(),
        ],
        if (type.showHours) ...[
          _buildDigit(
            timeStream,
            (DateTime? time) => (timeCounter.inHours % 24) ~/ 10,
            (DateTime? time) => (timeCounter.inHours % 24) % 10,
            DateTime.now(),
            "Hours",
          ),
          if (widget.separator.isNotEmpty) _buildSeparator(),
        ],
        if (type.showMinutes) ...[
          _buildDigit(
            timeStream,
            (DateTime? time) => (timeCounter.inMinutes % 60) ~/ 10,
            (DateTime? time) => (timeCounter.inMinutes % 60) % 10,
            DateTime.now(),
            "minutes",
          ),
          if (widget.separator.isNotEmpty) _buildSeparator(),
        ],
        _buildDigit(
          timeStream,
          (DateTime? time) => (timeCounter.inSeconds % 60) ~/ 10,
          (DateTime? time) => (timeCounter.inSeconds % 60) % 10,
          DateTime.now(),
          "seconds",
        )
      ],
    );
  }

  Widget _buildSeparator() {
    return Text(
      widget.separator,
      textAlign: TextAlign.center,
      style: widget.separatorTextStyle ?? widget.textStyle,
    );
  }

  Widget _buildDigitForLargeNumber(
    Stream<DateTime?> timeStream,
    List<DigitFunction> digits,
    DateTime startTime,
    String id,
  ) {
    String timeLeftString = timeCounter.inDays.toString();
    List<Widget> rows = [];
    for (int i = 0; i < timeLeftString.toString().length; i++) {
      rows.add(
        Container(
          decoration: widget.decoration,
          padding: widget.tightLabel
              ? const EdgeInsets.only(left: 3)
              : EdgeInsets.zero,
          child: Digit(
            padding: widget.padding,
            itemStream: timeStream.map<int>(digits[i]),
            initValue: digits[i](startTime),
            decoration: widget.decoration,
            slideDirection: slideDirection,
            textStyle: widget.textStyle,
          ),
        ),
      );
    }

    return Column(
      mainAxisSize: MainAxisSize.min,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: <Widget>[
        Row(
          mainAxisSize: MainAxisSize.min,
          mainAxisAlignment: MainAxisAlignment.center,
          children: rows,
        ),
      ],
    );
  }

  Widget _buildDigit(
    Stream<DateTime?> timeStream,
    DigitFunction tensDigit,
    DigitFunction onesDigit,
    DateTime startTime,
    String id,
  ) {
    return Column(
      mainAxisSize: MainAxisSize.min,
      crossAxisAlignment: CrossAxisAlignment.center,
      children: <Widget>[
        Row(
          mainAxisSize: MainAxisSize.min,
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            Container(
              decoration: widget.decoration,
              padding: widget.tightLabel
                  ? const EdgeInsets.only(left: 3)
                  : EdgeInsets.zero,
              child: Digit(
                padding: widget.padding,
                itemStream: timeStream.map<int>(tensDigit),
                initValue: tensDigit(startTime),
                decoration: widget.decoration,
                slideDirection: slideDirection,
                textStyle: widget.textStyle,
              ),
            ),
            Container(
              decoration: widget.decoration,
              padding: widget.tightLabel
                  ? const EdgeInsets.only(right: 3)
                  : EdgeInsets.zero,
              child: Digit(
                padding: widget.padding,
                itemStream: timeStream.map<int>(onesDigit),
                initValue: onesDigit(startTime),
                decoration: widget.decoration,
                slideDirection: slideDirection,
                textStyle: widget.textStyle,
              ),
            ),
          ],
        ),
      ],
    );
  }

  @override
  void dispose() {
    isDisposed = true;
    super.dispose();
  }
}
